package com.sun.jndi.toolkit.ctx;

import javax.naming.LinkRef;
import javax.naming.CompositeName;
import javax.naming.InvalidNameException;
import javax.naming.CannotProceedException;
import javax.naming.NamingException;
import javax.naming.Context;
import java.util.Hashtable;
import javax.naming.Name;
import javax.naming.spi.ResolveResult;

public class Continuation extends ResolveResult
{
    protected Name starter;
    protected Object followingLink;
    protected Hashtable<?, ?> environment;
    protected boolean continuing;
    protected Context resolvedContext;
    protected Name relativeResolvedName;
    private static final long serialVersionUID = 8162530656132624308L;
    
    public Continuation() {
        this.followingLink = null;
        this.environment = null;
        this.continuing = false;
        this.resolvedContext = null;
        this.relativeResolvedName = null;
    }
    
    public Continuation(Name starter, Hashtable<?, ?> hashtable) {
        this.followingLink = null;
        this.environment = null;
        this.continuing = false;
        this.resolvedContext = null;
        this.relativeResolvedName = null;
        this.starter = starter;
        this.environment = (Hashtable)(Hashtable)((hashtable == null) ? null : hashtable.clone());
    }
    
    public boolean isContinue() {
        return this.continuing;
    }
    
    public void setSuccess() {
        this.continuing = false;
    }
    
    public NamingException fillInException(NamingException ex) {
        ex.setRemainingName(this.remainingName);
        ex.setResolvedObj(this.resolvedObj);
        if (this.starter == null || this.starter.isEmpty()) {
            ex.setResolvedName(null);
        }
        else if (this.remainingName == null) {
            ex.setResolvedName(this.starter);
        }
        else {
            ex.setResolvedName(this.starter.getPrefix(this.starter.size() - this.remainingName.size()));
        }
        if ((ex instanceof CannotProceedException)) {
            CannotProceedException ex2 = (CannotProceedException)ex;
            ex2.setEnvironment((this.environment == null) ? new Hashtable(11) : ((Hashtable)this.environment.clone()));
            ex2.setAltNameCtx(this.resolvedContext);
            ex2.setAltName(this.relativeResolvedName);
            return ex;
        }
        return ex;
    }
    
    public void setErrorNNS(Object o, Name name) {
        Name name2 = (Name)(Name)name.clone();
        try {
            name2.add("");
        }
        catch (InvalidNameException ex) {}
        this.setErrorAux(o, name2);
    }
    
    public void setErrorNNS(Object o, String comp) {
        CompositeName compositeName = new CompositeName();
        try {
            if (comp != null && !comp.equals("")) {
                compositeName.add(comp);
            }
            compositeName.add("");
        }
        catch (InvalidNameException ex) {}
        this.setErrorAux(o, compositeName);
    }
    
    public void setError(Object o, Name name) {
        if (name != null) {
            this.remainingName = (Name)(Name)name.clone();
        }
        else {
            this.remainingName = null;
        }
        this.setErrorAux(o, this.remainingName);
    }
    
    public void setError(Object o, String comp) {
        CompositeName compositeName = new CompositeName();
        if (comp != null && !comp.equals("")) {
            try {
                compositeName.add(comp);
            }
            catch (InvalidNameException ex) {}
        }
        this.setErrorAux(o, compositeName);
    }
    
    private void setErrorAux(Object resolvedObj, Name remainingName) {
        this.remainingName = remainingName;
        this.resolvedObj = resolvedObj;
        this.continuing = false;
    }
    
    private void setContinueAux(Object resolvedObj, Name relativeResolvedName, Context resolvedContext, Name remainingName) {
        if ((resolvedObj instanceof LinkRef)) {
            this.setContinueLink(resolvedObj, relativeResolvedName, resolvedContext, remainingName);
        }
        else {
            this.remainingName = remainingName;
            this.resolvedObj = resolvedObj;
            this.relativeResolvedName = relativeResolvedName;
            this.resolvedContext = resolvedContext;
            this.continuing = true;
        }
    }
    
    public void setContinueNNS(Object o, Name name, Context context) {
        CompositeName compositeName = new CompositeName();
        this.setContinue(o, name, context, PartialCompositeContext._NNS_NAME);
    }
    
    public void setContinueNNS(Object o, String comp, Context context) {
        CompositeName compositeName = new CompositeName();
        try {
            compositeName.add(comp);
        }
        catch (NamingException ex) {}
        this.setContinue(o, compositeName, context, PartialCompositeContext._NNS_NAME);
    }
    
    public void setContinue(Object o, Name name, Context context) {
        this.setContinueAux(o, name, context, (Name)PartialCompositeContext._EMPTY_NAME.clone());
    }
    
    public void setContinue(Object o, Name name, Context context, Name name2) {
        if (name2 != null) {
            this.remainingName = (Name)(Name)name2.clone();
        }
        else {
            this.remainingName = new CompositeName();
        }
        this.setContinueAux(o, name, context, this.remainingName);
    }
    
    public void setContinue(Object o, String comp, Context context, String comp2) {
        CompositeName compositeName = new CompositeName();
        if (!comp.equals("")) {
            try {
                compositeName.add(comp);
            }
            catch (NamingException ex) {}
        }
        CompositeName compositeName2 = new CompositeName();
        if (!comp2.equals("")) {
            try {
                compositeName2.add(comp2);
            }
            catch (NamingException ex2) {}
        }
        this.setContinueAux(o, compositeName, context, compositeName2);
    }
    
    @Deprecated
    public void setContinue(Object o, Object o2) {
        this.setContinue(o, null, (Context)o2);
    }
    
    private void setContinueLink(Object followingLink, Name name, Context context, Name remainingName) {
        this.followingLink = followingLink;
        this.remainingName = remainingName;
        this.resolvedObj = context;
        this.relativeResolvedName = PartialCompositeContext._EMPTY_NAME;
        this.resolvedContext = context;
        this.continuing = true;
    }
    
    @Override
    public String toString() {
        if (this.remainingName != null) {
            return this.starter.toString() + "; remainingName: '" + this.remainingName + "'";
        }
        return this.starter.toString();
    }
    
    public String toString(boolean b) {
        if (!b || this.resolvedObj == null) {
            return this.toString();
        }
        return this.toString() + "; resolvedObj: " + this.resolvedObj + "; relativeResolvedName: " + this.relativeResolvedName + "; resolvedContext: " + this.resolvedContext;
    }
}
